// Command code and error information for I2C driver
// (For convenience in using just OS_loaderInit, these are included here.  The "master"
//  copies are considered the ones in OS_loader and these must be kept up-to-date).
// Based on work of Mike Green
// transscripted to C by Nick Mueller
//
// *** NOTE:
// This C-port is more a brutal hack then a nice port.
// I hacked the ASM to make it smaller (removed most but not all SPI-code) 
// Some useless defines might be left over.
// Some of the original comments were left.
// But at least, it can read and write to EEPROM.
//
// Tested with Microchip 24xx512 (64k)
//
// change comment of ioLowSpeed for fast (400kHz) / slow EEPROM

#include <propeller.h>
#include "i2c.h"
#include "i2ccAsmS_array.h"

#define ioReadCmd   0b00000001
#define ioWriteCmd  0b00000010
#define ioRead1Cmd  0b00000011
#define ioWrite1Cmd 0b00000100
#define ioRead0Cmd  0b00000101
#define ioWrite0Cmd 0b00000110
#define ioBootCmd   0b00001000
#define ioCmdMask   0b00011111
#define ioNoStore   0b00100000

// comment/uncomment next two lines for speed
#define ioLowSpeed  0b00000000
// #define  ioLowSpeed  0b01000000

#define ioStopLdr   0b10000000
#define ioWriteErr  0b10000000

#define ioTestRdy   (ioCmdMask << 24)
#define ioTestErr   (ioWriteErr << 24)

#define clkfreqVal  0x0000
#define clksetVal   0x0004
#define chksumVal   0x0005
#define vbase       0x0008
#define dbase       0x000A


int i2cCog = 0;
i2cControlBlock_t *pControl;
// ------------------------------------------------
int i2cIO(long command, int addr, void* pBuffer, long count) {
  while ((*pControl)[0] & ioTestRdy) {}                       // Wait for previous I/O to finish
  (*pControl)[1] = (count << 16) | ((long)pBuffer & 0xFFFF);
  (*pControl)[0] = (command << 24) | (addr & 0xFFFFFF);
  while ((*pControl)[0] & ioTestRdy) {}                       // Wait for this to finish
  return ((*pControl)[0] & ioTestErr);                        // Return any error code
}
// ------------------------------------------------
int i2cReadEEPROM(int addr, void* pBuffer, long count) {      // Read a block from EEPROM to RAM
  return i2cIO(ioReadCmd | ioLowSpeed, addr, pBuffer, count);
}
// ------------------------------------------------
int i2cWriteEEPROM(int addr, void* pBuffer, long count) {     // Write a block to EEPROM from RAM
  return i2cIO(ioWriteCmd | ioLowSpeed, addr, pBuffer, count);
}
// ------------------------------------------------
// This routine checks to be sure there is an I2C bus and an EEPROM at the
// specified address.  Note that this routine cannot distinguish between a
// 32Kx8 and a 64Kx8 EEPROM since the 16th address bit is a "don't care"
// for the 32Kx8 devices.  Return true if EEPROM present, false otherwise.

int i2cCheckPresence(int addr) {
  return i2cIO(ioReadCmd | ioLowSpeed, addr, NULL, 0) == 0;
}
// ------------------------------------------------
int i2cWriteWait(int addr) {
  int t;                                                  // Wait for EEPROM to complete write
  
  t = CNT;
  do {
    if ((CNT - t) > (CLKFREQ / 50))
      return TRUE;                                      // Return true if a timeout occurred
  } while (!i2cCheckPresence(addr));                    // Maximum wait time is 20ms
  return FALSE;                                         // Otherwise return false
}
// ------------------------------------------------
int i2cStart(i2cControlBlock_t* pCtrlBlk) {
  pControl = pCtrlBlk;
  i2cStop();                                              // Stop a previous copy
  (*pControl)[0] = 0;                                     // Initialize the control block
  (*pControl)[1] = 0;
  
  i2cCog = cognew_native((void*)i2ccAsmS_array, (void*)pControl) + 1;           //' Start a new cog with the I2C driver
  return i2cCog > 0;                                      // Indicate success
}
// ------------------------------------------------
void i2cStop(void) {
  int tCog = i2cCog - 1;                                  // Stop the I2C I/O driver (standalone)
  
  if (i2cCog > 0)
    COGSTOP(tCog);
}

